import { getHashString } from "./utils";

export async function downloadDispatcher(request, env, ctx) {
    const url = new URL(request.url);
    const uri = url.pathname;

    const bucket = env.EXP_DATA;
    const signatureBase = env.SIGNATURE_BASE;
    const adminSecret = env.ADMIN_SECRET;

    const key = uri.slice(10); // '/download/'.length
    const signature = url.searchParams.get('signature');
    const timestamp = url.searchParams.get('timestamp');
    
    if (!signature && request.headers.get('x-access-token') !== adminSecret)
        return new Response(null, { status: 403, statusText: 'Forbidden' });

    const hash = await getHashString(`${key}${timestamp}${signatureBase}`, 'SHA-1');
    if (signature !== hash)
        return new Response(null, { status: 403, statusText: 'Forbidden' });
    if (Date.now() - Number(timestamp) > 1000 * 60 * 5) // 5 minutes
        return new Response(null, { status: 403, statusText: 'Signature Expired' });

    try {
        const object = await bucket.get(key);
        if (object === null)
            return new Response(null, { status: 404, statusText: 'Not Found' });

        const { body, etag, httpMetadata, customMetadata } = object;
        return new Response(body, { headers: {
            etag,
            'content-type': httpMetadata.contentType,
            'content-length': body.length,
            'x-metadata': JSON.stringify(customMetadata),
        }});

    } catch (e) { return new Response(null, { status: 500 }); }
}

export default downloadDispatcher;