import { verifyAccessToken } from "./getAccessToken";

const allowedContentTypes = [
	'text/plain',
	'text/csv',
    'application/json',
];

async function invalidPutRequest(request, env, ctx) {
    const headers = request.headers;
	const subid = headers.get('x-subject-id');
	const contentType = headers.get('content-type');
	
	return (subid === null)
        || (!allowedContentTypes.includes(contentType))
		|| (!await verifyAccessToken(request, env, ctx));
}

export async function putDispatcher(request, env, ctx) {
    const url = new URL(request.url);
    const uri = url.pathname;
    const headers = request.headers;
    
    const bucket = env.EXP_DATA;
    const keyPrefix = env.KEY_PREFIX;
    const timestamp = new Date().toISOString().replace(/:/g, '_');
    
    try {
        if (await invalidPutRequest(request, env, ctx))
            return new Response(null, { status: 401, statusText: 'Unauthorized' });
        
        const studyId = headers.get('x-study-id');
        const subjectId = headers.get('x-subject-id');
        const data = await request.text();
        const objectKey = `${keyPrefix}${studyId}/${subjectId}${uri}`;
        const customMetadata = JSON.parse(headers.get('x-metadata') || '{}');
        const httpMetadata = { contentType: headers.get('content-type') };
        const result = await bucket.put(
            objectKey, data,
            { customMetadata, httpMetadata }
        );
        
        return new Response(JSON.stringify({
            timestamp,
            objectKey,
            etag: result.etag,
            message: 'Data saved successfully',
        }), {
            status: 200,
            headers: {
                'Access-Control-Allow-Origin': headers.get('Origin'),
                'Access-Control-Allow-Methods': 'GET,PUT,OPTIONS',
                'Access-Control-Allow-Headers': '*',
            }
        });

    } catch (e) { console.log(e); return new Response(null, { status: 400 }); }
}

export default putDispatcher;